from dataclasses import asdict, dataclass, field
from typing import Union, Optional


@dataclass
class Entitlement:
    org_key: str
    external_id: str
    system: str
    origin: str
    status: str
    product_id: str
    config: dict = field(default_factory=dict)
    custom_attr: dict = field(default_factory=dict)
    org_uuid: Optional[Union[str, None]] = None
    begin_date: Optional[Union[str, None]] = None
    end_date: Optional[Union[str, None]] = None
    licenses: Optional[Union[int, None]] = None
    eval: Optional[Union[bool, None]] = None
    entitlement_id: Optional[str] = None

    def __post_init__(self):
        self.system = self.system.upper()
        self.origin = self.origin.upper()
        self.status = self.status.upper()
        # TOOLS-2801 concerns in the ticket
        # self.product_id = self.product_id.lower()

    def asdict(self):
        return asdict(self)


def new_entitlement(organization: dict, new: dict) -> Entitlement:
    entitlement = Entitlement(
        org_key=organization.get("org_key"),
        org_uuid=organization.get("csp_organization", {}).get("id"),
        external_id=new.get("external_urn"),
        product_id=new.get("product"),
        system=new.get("source"),
        origin=organization.get("origin"),
        status=new.get("status"),
        begin_date=new.get("begin_date"),
        end_date=new.get("end_date"),
        licenses=new.get("licenses"),
        eval=new.get("eval"),
        config={},
        custom_attr=new.get("config"),
    )

    return entitlement


def new_merged_entitlement(old: dict, new: dict) -> Entitlement:
    merged_entitlement = Entitlement(
        entitlement_id=old["entitlement_id"],
        org_key=old["org_key"],
        org_uuid=old.get("org_uuid") or None,
        external_id=new["external_id"],
        product_id=new["product_id"],
        system=new["system"],
        origin=new["origin"],
        status=new["status"],
        begin_date=new["begin_date"],
        end_date=new["end_date"],
        licenses=new["licenses"],
        eval=new["eval"],
        config=old["config"],
        custom_attr=new["custom_attr"],
    )

    return merged_entitlement
