import requests
from os import environ
from static.utils import Log
from requests.adapters import HTTPAdapter
from urllib3.util.retry import Retry

logger = Log()


class TimeoutHTTPAdapter(HTTPAdapter):
    def __init__(self, *args, **kwargs):
        self.timeout = int(environ.get("REQUESTS_TIMEOUT"))
        if "timeout" in kwargs:
            self.timeout = kwargs["timeout"]
            kwargs.pop("timeout", None)
        super().__init__(*args, **kwargs)

    def send(self, request, **kwargs):
        timeout = kwargs.get("timeout")
        if timeout is None:
            kwargs["timeout"] = self.timeout
        return super().send(request, **kwargs)


def print_resp(response, *args, **kwargs):
    logger.send_info(
        {
            "status_code": response.status_code,
            "endpoint": response.url,
            "method": response.request.method,
            "request_body": response.request.__dict__["body"],
            "response_body": response.text,
        }
    )


def assert_status_hook(response, *args, **kwargs):
    response.raise_for_status()


def requests_retry_session(
    retries=int(environ.get("REQUESTS_RETRY")),
    backoff_factor=int(environ.get("REQUESTS_BACKOFF")),
    status_forcelist=(429, 500, 502, 503, 504),
    session=None,
):
    session = session or requests.Session()
    retry = Retry(
        total=retries,
        read=retries,
        connect=retries,
        backoff_factor=backoff_factor,
        status_forcelist=status_forcelist,
    )

    adapter = TimeoutHTTPAdapter(max_retries=retry)

    session.hooks["response"] = [print_resp, assert_status_hook]
    session.mount("http://", adapter)
    session.mount("https://", adapter)

    return session
